package api

import (
	"errors"
	"sku3dweb/db/model"
	"sku3dweb/db/repo"
	"sku3dweb/log"
	"time"

	"github.com/gin-gonic/gin"
	"go.mongodb.org/mongo-driver/bson"
	"go.mongodb.org/mongo-driver/bson/primitive"
)

func FassiApi(r *GinRouter) {
	r.POSTJWT("/image/create", createImg)
	r.GETJWT("/image/fassi/list", SearchByImg)
	r.GETJWT("/image/list", SearchByFields)
	r.POSTJWT("/image/delete/:id", DeleteImage)
	r.POSTJWT("/image/update", UpdateImage)
}

func UpdateImage(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	var matImage model.MatImage
	err := c.ShouldBindJSON(&matImage)
	if err != nil {
		log.Error(err)
		return nil, err
	}
	if matImage.Id.IsZero() {
		return nil, errors.New("id错误")
	}
	searchMat := &model.MatImage{}
	// 如果更新的是面料图
	found, err := repo.RepoSeachDoc(apictx.CreateRepoCtx(), &repo.DocSearchOptions{
		CollectName: repo.CollectionMatImages,
		Query:       repo.Map{"_id": matImage.Id},
		Project:     []string{"rawImage"},
	}, &searchMat)
	if err != nil {
		return nil, err
	}
	// 没有找到面料数据
	if !found {
		return nil, NewError("未找到数据")
	}
	// 更新了面料原图 对应更新fassi 特征数据
	if searchMat.RawImage.Url != matImage.RawImage.Url {
		// 先删除
		_, err := RomoveFassiImage(matImage.Id.Hex())
		if err != nil {
			return nil, err
		}
		// 再新增
		err = AddFassiImage(matImage.Id, matImage.RawImage.Url)
		if err != nil {
			return nil, err
		}
	}

	return repo.RepoUpdateSetDoc(apictx.CreateRepoCtx(), repo.CollectionMatImages, matImage.Id.Hex(), &matImage)
}

func DeleteImage(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	id := c.Param("id")
	_, err := RomoveFassiImage(id)
	if err != nil {
		return nil, err
	}
	return repo.RepoDeleteDoc(apictx.CreateRepoCtx(), repo.CollectionMatImages, id)
}

func SearchByFields(c *gin.Context, apictx *ApiSession) (interface{}, error) {
	page, size, query := UtilQueryPageSize(c)

	pageOption := &repo.PageSearchOptions{
		CollectName: repo.CollectionMatImages,
		Page:        page,
		Size:        size,
		Query:       query,
		Project:     []string{"nameCn", "createTime", "thumbnail", "price", "from"},
		// Sort: bson.M{"_id"},
	}
	return repo.RepoPageSearch(apictx.CreateRepoCtx(), pageOption)
}

func createImg(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	data := &model.MatImage{}
	err := c.ShouldBindJSON(data)
	if err != nil {
		return nil, err
	}
	data.CreateTime = time.Now()
	data.UpdateTime = time.Now()
	data.UserId, _ = primitive.ObjectIDFromHex(apictx.User.ID)

	idstr, err := repo.RepoAddDoc(apictx.CreateRepoCtx(), repo.CollectionMatImages, data)
	if err != nil {
		return nil, err
	}
	imgId, _ := primitive.ObjectIDFromHex(idstr)

	return imgId, AddFassiImage(imgId, data.RawImage.Url)
}

func SearchByImg(c *gin.Context, apictx *ApiSession) (interface{}, error) {

	data := &struct {
		Url string
	}{}
	err := c.ShouldBindJSON(data)
	if err != nil || len(data.Url) < 1 {
		return nil, NewError("参数错误")
	}

	images, err := QueryFassiImage(data.Url)
	if err != nil {
		return nil, err
	}

	var list = []map[string]interface{}{}
	if len(images) < 1 {
		return list, nil
	}

	var ids = []primitive.ObjectID{}
	for _, item := range images {
		ids = append(ids, item.Id)

	}
	param := &repo.DocSearchOptions{
		CollectName: repo.CollectionMatImages,
		Query:       repo.Map{"_id": bson.M{"$in": ids}},
	}
	err = repo.RepoSeachDocs(apictx.CreateRepoCtx(), param, &list)
	if err != nil {
		return nil, err
	}

	var ImageMap = map[primitive.ObjectID]map[string]interface{}{}
	for _, item := range list {
		ImageMap[item["_id"].(primitive.ObjectID)] = item
	}

	var out = []interface{}{}
	for _, item := range images {
		value := ImageMap[item.Id]
		value["score"] = item.Score
		out = append(out, value)
	}
	return out, nil
}
